@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2009 - 2017 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2009 - 2018 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -13,7 +13,7 @@ import re
 import json
 
 from PyQt5.QtCore import pyqtSignal, pyqtSlot, QLocale, QUrl, QByteArray, \
-    QBuffer, QIODevice, QObject, qVersion
+    QBuffer, QIODevice, QObject
 from PyQt5.QtGui import QImage
 from PyQt5.QtNetwork import QNetworkRequest, QNetworkAccessManager, \
     QNetworkReply
@@ -22,6 +22,7 @@ from UI.Info import Program
 
 import Preferences
 import Utilities
+from Globals import qVersionTuple
 
 
 class OpenSearchEngine(QObject):
@@ -179,7 +180,7 @@ class OpenSearchEngine(QObject):
             .encode("utf-8"))
         
         if self.__searchMethod != "post":
-            if qVersion() >= "5.0.0":
+            if qVersionTuple() >= (5, 0, 0):
                 from PyQt5.QtCore import QUrlQuery
                 urlQuery = QUrlQuery(ret)
                 for parameter in self._searchParameters:
@@ -234,7 +235,7 @@ class OpenSearchEngine(QObject):
             searchTerm, self._suggestionsUrlTemplate).encode("utf-8")))
         
         if self.__searchMethod != "post":
-            if qVersion() >= "5.0.0":
+            if qVersionTuple() >= (5, 0, 0):
                 from PyQt5.QtCore import QUrlQuery
                 urlQuery = QUrlQuery(ret)
                 for parameter in self._suggestionsParameters:
