@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2007 - 2015 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2007 - 2018 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -10,9 +10,10 @@ Module implementing a dialog to generate code for a Qt4/Qt5 dialog.
 from __future__ import unicode_literals
 
 import os
+import xml.etree.ElementTree
 
 from PyQt5.QtCore import QMetaObject, QByteArray, QRegExp, Qt, pyqtSlot, \
-    QMetaMethod, qVersion, QSortFilterProxyModel
+    QMetaMethod, QSortFilterProxyModel
 from PyQt5.QtGui import QStandardItemModel, QBrush, QStandardItem
 from PyQt5.QtWidgets import QWidget, QDialog, QDialogButtonBox, QAction
 from PyQt5 import uic
@@ -26,9 +27,15 @@ from .NewDialogClassDialog import NewDialogClassDialog
 
 from eric6config import getConfig
 
+from Globals import qVersionTuple
+
+
 pyqtSignatureRole = Qt.UserRole + 1
 pythonSignatureRole = Qt.UserRole + 2
 rubySignatureRole = Qt.UserRole + 3
+returnTypeRole = Qt.UserRole + 4
+parameterTypesListRole = Qt.UserRole + 5
+parameterNamesListRole = Qt.UserRole + 6
 
 
 class CreateDialogCodeDialog(QDialog, Ui_CreateDialogCodeDialog):
@@ -142,9 +149,11 @@ class CreateDialogCodeDialog(QDialog, Ui_CreateDialogCodeDialog):
         @return object name (string)
         """
         try:
-            dlg = uic.loadUi(self.formFile)
+            dlg = uic.loadUi(
+                self.formFile, package=self.project.getProjectPath())
             return dlg.objectName()
-        except (AttributeError, ImportError) as err:
+        except (AttributeError, ImportError,
+                xml.etree.ElementTree.ParseError) as err:
             E5MessageBox.critical(
                 self,
                 self.tr("uic error"),
@@ -160,9 +169,11 @@ class CreateDialogCodeDialog(QDialog, Ui_CreateDialogCodeDialog):
         @return class name (sting)
         """
         try:
-            dlg = uic.loadUi(self.formFile)
+            dlg = uic.loadUi(
+                self.formFile, package=self.project.getProjectPath())
             return dlg.metaObject().className()
-        except (AttributeError, ImportError) as err:
+        except (AttributeError, ImportError,
+                xml.etree.ElementTree.ParseError) as err:
             E5MessageBox.critical(
                 self,
                 self.tr("uic error"),
@@ -210,7 +221,7 @@ class CreateDialogCodeDialog(QDialog, Ui_CreateDialogCodeDialog):
             # 1. check for const
             mapped = mapped.replace("const ", "")
             
-            # 2. check fpr *
+            # 2. check for *
             mapped = mapped.replace("*", "")
             
             # 3. replace QString and QStringList
@@ -229,7 +240,8 @@ class CreateDialogCodeDialog(QDialog, Ui_CreateDialogCodeDialog):
         self.filterEdit.clear()
         
         try:
-            dlg = uic.loadUi(self.formFile)
+            dlg = uic.loadUi(
+                self.formFile, package=self.project.getProjectPath())
             objects = dlg.findChildren(QWidget) + dlg.findChildren(QAction)
             
             signatureList = self.__signatures()
@@ -238,7 +250,8 @@ class CreateDialogCodeDialog(QDialog, Ui_CreateDialogCodeDialog):
             self.slotsModel.setHorizontalHeaderLabels([""])
             for obj in objects:
                 name = obj.objectName()
-                if not name:
+                if not name or name.startswith("qt_"):
+                    # ignore un-named or internal objects
                     continue
                 
                 metaObject = obj.metaObject()
@@ -248,7 +261,7 @@ class CreateDialogCodeDialog(QDialog, Ui_CreateDialogCodeDialog):
                 for index in range(metaObject.methodCount()):
                     metaMethod = metaObject.method(index)
                     if metaMethod.methodType() == QMetaMethod.Signal:
-                        if qVersion() >= "5.0.0":
+                        if qVersionTuple() >= (5, 0, 0):
                             itm2 = QStandardItem("on_{0}_{1}".format(
                                 name,
                                 bytes(metaMethod.methodSignature()).decode()))
@@ -257,7 +270,7 @@ class CreateDialogCodeDialog(QDialog, Ui_CreateDialogCodeDialog):
                                 name, metaMethod.signature()))
                         itm.appendRow(itm2)
                         if self.__module is not None:
-                            if qVersion() >= "5.0.0":
+                            if qVersionTuple() >= (5, 0, 0):
                                 method = "on_{0}_{1}".format(
                                     name,
                                     bytes(metaMethod.methodSignature())
@@ -277,22 +290,28 @@ class CreateDialogCodeDialog(QDialog, Ui_CreateDialogCodeDialog):
                                 itm2.setForeground(QBrush(Qt.blue))
                                 continue
                         
-                        pyqtSignature = \
-                            ", ".join([self.__mapType(t)
-                                       for t in metaMethod.parameterTypes()])
+                        returnType = self.__mapType(
+                            metaMethod.typeName().encode())
+                        if returnType == 'void':
+                            returnType = ""
+                        parameterTypesList = [
+                            self.__mapType(t)
+                            for t in metaMethod.parameterTypes()]
+                        pyqtSignature = ", ".join(parameterTypesList)
                         
                         parameterNames = metaMethod.parameterNames()
                         if parameterNames:
                             for index in range(len(parameterNames)):
                                 if not parameterNames[index]:
                                     parameterNames[index] = \
-                                        QByteArray("p{0:d}".format(index))
-                        methNamesSig = \
-                            ", ".join(
-                                [bytes(n).decode() for n in parameterNames])
+                                        QByteArray("p{0:d}".format(index)
+                                                   .encode("utf-8"))
+                        parameterNamesList = [bytes(n).decode()
+                                              for n in parameterNames]
+                        methNamesSig = ", ".join(parameterNamesList)
                         
                         if methNamesSig:
-                            if qVersion() >= "5.0.0":
+                            if qVersionTuple() >= (5, 0, 0):
                                 pythonSignature = \
                                     "on_{0}_{1}(self, {2})".format(
                                         name,
@@ -306,7 +325,7 @@ class CreateDialogCodeDialog(QDialog, Ui_CreateDialogCodeDialog):
                                         metaMethod.signature().split("(")[0],
                                         methNamesSig)
                         else:
-                            if qVersion() >= "5.0.0":
+                            if qVersionTuple() >= (5, 0, 0):
                                 pythonSignature = "on_{0}_{1}(self)".format(
                                     name,
                                     bytes(metaMethod.methodSignature())
@@ -317,6 +336,11 @@ class CreateDialogCodeDialog(QDialog, Ui_CreateDialogCodeDialog):
                                     metaMethod.signature().split("(")[0])
                         itm2.setData(pyqtSignature, pyqtSignatureRole)
                         itm2.setData(pythonSignature, pythonSignatureRole)
+                        itm2.setData(returnType, returnTypeRole)
+                        itm2.setData(parameterTypesList,
+                                     parameterTypesListRole)
+                        itm2.setData(parameterNamesList,
+                                     parameterNamesListRole)
                         
                         itm2.setFlags(Qt.ItemFlags(
                             Qt.ItemIsUserCheckable |
@@ -326,7 +350,8 @@ class CreateDialogCodeDialog(QDialog, Ui_CreateDialogCodeDialog):
                         itm2.setCheckState(Qt.Unchecked)
             
             self.slotsView.sortByColumn(0, Qt.AscendingOrder)
-        except (AttributeError, ImportError) as err:
+        except (AttributeError, ImportError,
+                xml.etree.ElementTree.ParseError) as err:
             E5MessageBox.critical(
                 self,
                 self.tr("uic error"),
@@ -486,15 +511,34 @@ class CreateDialogCodeDialog(QDialog, Ui_CreateDialogCodeDialog):
                             child.data(pyqtSignatureRole))))
                     slotsCode.append('{0}def {1}:\n'.format(
                         indentStr, child.data(pythonSignatureRole)))
-                    slotsCode.append('{0}"""\n'.format(indentStr * 2))
+                    indentStr2 = indentStr * 2
+                    slotsCode.append('{0}"""\n'.format(indentStr2))
                     slotsCode.append(
                         '{0}Slot documentation goes here.\n'.format(
-                            indentStr * 2))
-                    slotsCode.append('{0}"""\n'.format(indentStr * 2))
+                            indentStr2))
+                    if child.data(returnTypeRole) or \
+                            child.data(parameterTypesListRole):
+                        slotsCode.append('{0}\n'.format(indentStr2))
+                        if child.data(parameterTypesListRole):
+                            for name, type_ in zip(
+                                child.data(parameterNamesListRole),
+                                    child.data(parameterTypesListRole)):
+                                slotsCode.append(
+                                    '{0}@param {1} DESCRIPTION\n'.format(
+                                        indentStr2, name))
+                                slotsCode.append('{0}@type {1}\n'.format(
+                                    indentStr2, type_))
+                        if child.data(returnTypeRole):
+                            slotsCode.append(
+                                '{0}@returns DESCRIPTION\n'.format(
+                                    indentStr2))
+                            slotsCode.append('{0}@rtype {1}\n'.format(
+                                indentStr2, child.data(returnTypeRole)))
+                    slotsCode.append('{0}"""\n'.format(indentStr2))
                     slotsCode.append('{0}# {1}: not implemented yet\n'.format(
-                        indentStr * 2, "TODO"))
+                        indentStr2, "TODO"))
                     slotsCode.append('{0}raise NotImplementedError\n'.format(
-                        indentStr * 2))
+                        indentStr2))
         
         if appendAtIndex == -1:
             sourceImpl.extend(slotsCode)
