@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2002 - 2016 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2002 - 2018 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -41,9 +41,6 @@ class ProjectInterfacesBrowser(ProjectBaseBrowser):
     """
     A class used to display the interfaces (IDL) part of the project.
     
-    @signal sourceFile(str, int = 0) emitted to open a file
-    @signal closeSourceWindow(str) emitted after a file has been
-        removed/deleted from the project
     @signal appendStdout(str) emitted after something was received from
         a QProcess on stdout
     @signal appendStderr(str) emitted after something was received from
@@ -486,8 +483,8 @@ class ProjectInterfacesBrowser(ProjectBaseBrowser):
             poaList = glob.glob(os.path.join(path, "*__POA"))
             npoaList = [f.replace("__POA", "") for f in poaList]
             fileList = glob.glob(os.path.join(path, "*_idl.py"))
-            for dir in poaList + npoaList:
-                fileList += Utilities.direntries(dir, True, "*.py")
+            for directory in poaList + npoaList:
+                fileList += Utilities.direntries(directory, True, "*.py")
             for file in fileList:
                 self.project.appendFile(file)
             if not self.noDialog and not ui.notificationsEnabled():
