@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2004 - 2015 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2004 - 2018 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -142,7 +142,7 @@ class QsciScintillaCompat(QsciScintilla):
         except AttributeError:
             rangeHigh = list(range(40, 128))
         
-        f = font.family().encode()
+        f = font.family().encode("utf-8")
         ps = font.pointSize()
         weight = -font.weight()
         italic = font.italic()
@@ -242,14 +242,6 @@ class QsciScintillaCompat(QsciScintilla):
         """
         self.SendScintilla(QsciScintilla.SCI_SETSTYLING, length, style)
     
-    def setStyleBits(self, bits):
-        """
-        Public method to set the number of bits to be used for styling.
-        
-        @param bits number of style bits (integer)
-        """
-        self.SendScintilla(QsciScintilla.SCI_SETSTYLEBITS, bits)
-    
     def charAt(self, pos):
         """
         Public method to get the character at a position in the text observing
@@ -1093,6 +1085,135 @@ class QsciScintillaCompat(QsciScintilla):
         if hasattr(QsciScintilla, "SCI_FINDINDICATORHIDE"):
             self.SendScintilla(QsciScintilla.SCI_FINDINDICATORHIDE)
     
+    def getIndicatorStartPos(self, indicator, pos):
+        """
+        Public method to get the start position of an indicator at a position.
+        
+        @param indicator ID of the indicator (integer)
+        @param pos position within the indicator (integer)
+        @return start position of the indicator (integer)
+        """
+        return self.SendScintilla(QsciScintilla.SCI_INDICATORSTART,
+                                  indicator, pos)
+    
+    def getIndicatorEndPos(self, indicator, pos):
+        """
+        Public method to get the end position of an indicator at a position.
+        
+        @param indicator ID of the indicator (integer)
+        @param pos position within the indicator (integer)
+        @return end position of the indicator (integer)
+        """
+        return self.SendScintilla(QsciScintilla.SCI_INDICATOREND,
+                                  indicator, pos)
+    
+    def gotoPreviousIndicator(self, indicator, wrap):
+        """
+        Public method to move the cursor to the previous position of an
+        indicator.
+        
+        This method ensures, that the position found is visible (i.e. unfolded
+        and inside the visible range). The text containing the indicator is
+        selected.
+        
+        @param indicator ID of the indicator to search (integer)
+        @param wrap flag indicating to wrap around at the beginning of the
+            text (boolean)
+        @return flag indicating if the indicator was found (boolean)
+        """
+        pos = self.SendScintilla(QsciScintilla.SCI_GETCURRENTPOS)
+        docLen = self.SendScintilla(QsciScintilla.SCI_GETTEXTLENGTH)
+        isInIndicator = self.hasIndicator(indicator, pos)
+        posStart = self.getIndicatorStartPos(indicator, pos)
+        posEnd = self.getIndicatorEndPos(indicator, pos)
+        
+        if posStart == 0 and posEnd == docLen - 1:
+            # indicator does not exist
+            return False
+        
+        if posStart <= 0:
+            if not wrap:
+                return False
+            
+            isInIndicator = self.hasIndicator(indicator, docLen - 1)
+            posStart = self.getIndicatorStartPos(indicator, docLen - 1)
+        
+        if isInIndicator:
+            # get out of it
+            posStart = self.getIndicatorStartPos(indicator, posStart - 1)
+            if posStart <= 0:
+                if not wrap:
+                    return False
+                
+                posStart = self.getIndicatorStartPos(indicator, docLen - 1)
+        
+        newPos = posStart - 1
+        posStart = self.getIndicatorStartPos(indicator, newPos)
+        posEnd = self.getIndicatorEndPos(indicator, newPos)
+        
+        if self.hasIndicator(indicator, posStart):
+            # found it
+            line, index = self.lineIndexFromPosition(posEnd)
+            self.ensureLineVisible(line)
+            self.SendScintilla(QsciScintilla.SCI_SETSEL, posEnd, posStart)
+            self.SendScintilla(QsciScintilla.SCI_SCROLLCARET)
+            return True
+        
+        return False
+    
+    def gotoNextIndicator(self, indicator, wrap):
+        """
+        Public method to move the cursor to the next position of an indicator.
+        
+        This method ensures, that the position found is visible (i.e. unfolded
+        and inside the visible range). The text containing the indicator is
+        selected.
+        
+        @param indicator ID of the indicator to search (integer)
+        @param wrap flag indicating to wrap around at the beginning of the
+            text (boolean)
+        @return flag indicating if the indicator was found (boolean)
+        """
+        pos = self.SendScintilla(QsciScintilla.SCI_GETCURRENTPOS)
+        docLen = self.SendScintilla(QsciScintilla.SCI_GETTEXTLENGTH)
+        isInIndicator = self.hasIndicator(indicator, pos)
+        posStart = self.getIndicatorStartPos(indicator, pos)
+        posEnd = self.getIndicatorEndPos(indicator, pos)
+        
+        if posStart == 0 and posEnd == docLen - 1:
+            # indicator does not exist
+            return False
+        
+        if posEnd >= docLen:
+            if not wrap:
+                return False
+            
+            isInIndicator = self.hasIndicator(indicator, 0)
+            posEnd = self.getIndicatorEndPos(indicator, 0)
+        
+        if isInIndicator:
+            # get out of it
+            posEnd = self.getIndicatorEndPos(indicator, posEnd)
+            if posEnd >= docLen:
+                if not wrap:
+                    return False
+                
+                posEnd = self.getIndicatorEndPos(indicator, 0)
+        
+        newPos = posEnd + 1
+        posStart = self.getIndicatorStartPos(indicator, newPos)
+        posEnd = self.getIndicatorEndPos(indicator, newPos)
+        
+        if self.hasIndicator(indicator, posStart):
+            # found it
+            line, index = self.lineIndexFromPosition(posEnd)
+            self.ensureLineVisible(line)
+            self.SendScintilla(QsciScintilla.SCI_SETSEL, posStart, posEnd)
+            self.SendScintilla(QsciScintilla.SCI_SCROLLCARET)
+            return True
+        
+        return False
+    
     ###########################################################################
     ## methods to perform folding related stuff
     ###########################################################################
@@ -1194,6 +1315,32 @@ class QsciScintillaCompat(QsciScintilla):
         self.SendScintilla(QsciScintilla.SCI_MARKERSETBACK,
                            QsciScintilla.SC_MARKNUM_FOLDEREND, backColor)
     
+    def getVisibleLineFromDocLine(self, docLine):
+        """
+        Public method to convert a document line number to a visible line
+        number (i.e. respect folded lines and annotations).
+        
+        @param docLine document line number to be converted
+        @type int
+        @return visible line number
+        @rtype int
+        """
+        return self.SendScintilla(QsciScintilla.SCI_VISIBLEFROMDOCLINE,
+                                  docLine)
+    
+    def getDocLineFromVisibleLine(self, displayLine):
+        """
+        Public method to convert a visible line number to a document line
+        number (i.e. respect folded lines and annotations).
+        
+        @param displayLine display line number to be converted
+        @type int
+        @return document line number
+        @rtype int
+        """
+        return self.SendScintilla(QsciScintilla.SCI_DOCLINEFROMVISIBLE,
+                                  displayLine)
+    
     ###########################################################################
     ## interface methods to the standard keyboard command set
     ###########################################################################
@@ -1303,28 +1450,28 @@ class QsciScintillaCompat(QsciScintilla):
     ###########################################################################
     
     if "showUserList" not in QsciScintilla.__dict__:
-        def showUserList(self, id, lst):
+        def showUserList(self, listId, lst):
             """
             Public method to show a user supplied list.
             
-            @param id id of the list (integer)
+            @param listId id of the list (integer)
             @param lst list to be show (list of strings)
             """
-            if id <= 0:
+            if listId <= 0:
                 return
             
             self.SendScintilla(
                 QsciScintilla.SCI_AUTOCSETSEPARATOR,
                 ord(self.UserSeparator))
             self.SendScintilla(
-                QsciScintilla.SCI_USERLISTSHOW, id,
+                QsciScintilla.SCI_USERLISTSHOW, listId,
                 self._encodeString(self.UserSeparator.join(lst)))
     
     ###########################################################################
     ## work-arounds for buggy behavior
     ###########################################################################
     
-    def __completionListSelected(self, id, txt):
+    def __completionListSelected(self, listId, txt):
         """
         Private slot to handle the selection from the completion list.
         
@@ -1333,11 +1480,20 @@ class QsciScintillaCompat(QsciScintilla):
         list but not giving it back when an item is selected via a
         double-click.
         
-        @param id the ID of the user list (integer)
+        @param listId the ID of the user list (integer)
         @param txt the selected text (string)
         """
         self.activateWindow()
     
+    def updateVerticalScrollBar(self):
+        """
+        Public method to update the vertical scroll bar to reflect the
+        additional lines added by annotations.
+        """
+        # Workaround because Scintilla.Redraw isn't implemented
+        self.SendScintilla(QsciScintilla.SCI_SETVSCROLLBAR, 0)
+        self.SendScintilla(QsciScintilla.SCI_SETVSCROLLBAR, 1)
+    
     ###########################################################################
     ## utility methods
     ###########################################################################
