@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2004 - 2016 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2004 - 2018 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -18,7 +18,7 @@ import re
 from difflib import _mdiff, IS_CHARACTER_JUNK
 
 from PyQt5.QtCore import QTimer, QEvent, pyqtSlot
-from PyQt5.QtGui import QColor, QFontMetrics, QBrush, QTextCursor
+from PyQt5.QtGui import QFontMetrics, QBrush, QTextCursor
 from PyQt5.QtWidgets import QWidget, QApplication, QDialogButtonBox
 
 from E5Gui import E5MessageBox
@@ -73,12 +73,12 @@ def sbsdiff(a, b, linenumberwidth=4):
             yield ('e', linenumberformat.format(ln1), l1,
                    linenumberformat.format(ln2), l2)
             continue
-        if ln2 == "" and l2 == "\n":
+        if ln2 == "" and l2 in ("\r\n", "\n", "\r"):
             yield ('d', linenumberformat.format(ln1), removeMarkers(l1),
-                   emptylineno, '\n')
+                   emptylineno, l2)
             continue
-        if ln1 == "" and l1 == "\n":
-            yield ('i', emptylineno, '\n',
+        if ln1 == "" and l1 in ("\r\n", "\n", "\r"):
+            yield ('i', emptylineno, l1,
                    linenumberformat.format(ln2), removeMarkers(l2))
             continue
         yield ('r', linenumberformat.format(ln1), l1,
@@ -90,7 +90,7 @@ class CompareDialog(QWidget, Ui_CompareDialog):
     Class implementing a dialog to compare two files and show the result side
     by side.
     """
-    def __init__(self, files=[], parent=None):
+    def __init__(self, files=None, parent=None):
         """
         Constructor
         
@@ -101,6 +101,9 @@ class CompareDialog(QWidget, Ui_CompareDialog):
         super(CompareDialog, self).__init__(parent)
         self.setupUi(self)
         
+        if files is None:
+            files = []
+        
         self.file1Picker.setMode(E5PathPickerModes.OpenFileMode)
         self.file2Picker.setMode(E5PathPickerModes.OpenFileMode)
         
@@ -130,20 +133,7 @@ class CompareDialog(QWidget, Ui_CompareDialog):
         
         self.on_synchronizeCheckBox_toggled(True)
         
-        font = Preferences.getEditorOtherFonts("MonospacedFont")
-        self.contents_1.setFontFamily(font.family())
-        self.contents_1.setFontPointSize(font.pointSize())
-        self.contents_2.setFontFamily(font.family())
-        self.contents_2.setFontPointSize(font.pointSize())
-        self.fontHeight = QFontMetrics(self.contents_1.currentFont()).height()
-        
-        self.cNormalFormat = self.contents_1.currentCharFormat()
-        self.cInsertedFormat = self.contents_1.currentCharFormat()
-        self.cInsertedFormat.setBackground(QBrush(QColor(190, 237, 190)))
-        self.cDeletedFormat = self.contents_1.currentCharFormat()
-        self.cDeletedFormat.setBackground(QBrush(QColor(237, 190, 190)))
-        self.cReplacedFormat = self.contents_1.currentCharFormat()
-        self.cReplacedFormat.setBackground(QBrush(QColor(190, 190, 237)))
+        self.__generateFormats()
         
         # connect some of our widgets explicitly
         self.file1Picker.textChanged.connect(self.__fileChanged)
@@ -168,7 +158,29 @@ class CompareDialog(QWidget, Ui_CompareDialog):
         else:
             self.file1Label.hide()
             self.file2Label.hide()
-
+    
+    def __generateFormats(self):
+        """
+        Private method to generate the various text formats.
+        """
+        font = Preferences.getEditorOtherFonts("MonospacedFont")
+        self.contents_1.setFontFamily(font.family())
+        self.contents_1.setFontPointSize(font.pointSize())
+        self.contents_2.setFontFamily(font.family())
+        self.contents_2.setFontPointSize(font.pointSize())
+        self.fontHeight = QFontMetrics(self.contents_1.currentFont()).height()
+        
+        self.cNormalFormat = self.contents_1.currentCharFormat()
+        self.cInsertedFormat = self.contents_1.currentCharFormat()
+        self.cInsertedFormat.setBackground(
+            QBrush(Preferences.getDiffColour("AddedColor")))
+        self.cDeletedFormat = self.contents_1.currentCharFormat()
+        self.cDeletedFormat.setBackground(
+            QBrush(Preferences.getDiffColour("RemovedColor")))
+        self.cReplacedFormat = self.contents_1.currentCharFormat()
+        self.cReplacedFormat.setBackground(
+            QBrush(Preferences.getDiffColour("ReplacedColor")))
+    
     def show(self, filename=None):
         """
         Public slot to show the dialog.
@@ -179,20 +191,21 @@ class CompareDialog(QWidget, Ui_CompareDialog):
             self.file1Picker.setText(filename)
         super(CompareDialog, self).show()
         
-    def __appendText(self, pane, linenumber, line, format, interLine=False):
+    def __appendText(self, pane, linenumber, line, charFormat,
+                     interLine=False):
         """
         Private method to append text to the end of the contents pane.
         
         @param pane text edit widget to append text to (QTextedit)
         @param linenumber number of line to insert (string)
         @param line text to insert (string)
-        @param format text format to be used (QTextCharFormat)
+        @param charFormat text format to be used (QTextCharFormat)
         @param interLine flag indicating interline changes (boolean)
         """
         tc = pane.textCursor()
         tc.movePosition(QTextCursor.End)
         pane.setTextCursor(tc)
-        pane.setCurrentCharFormat(format)
+        pane.setCurrentCharFormat(charFormat)
         if interLine:
             pane.insertPlainText("{0} ".format(linenumber))
             for txt in re.split(self.markerPattern, line):
@@ -202,7 +215,7 @@ class CompareDialog(QWidget, Ui_CompareDialog):
                         tc = pane.textCursor()
                         tc.movePosition(QTextCursor.End)
                         pane.setTextCursor(tc)
-                        pane.setCurrentCharFormat(format)
+                        pane.setCurrentCharFormat(charFormat)
                         pane.insertPlainText(txt1)
                     tc = pane.textCursor()
                     tc.movePosition(QTextCursor.End)
@@ -292,6 +305,8 @@ class CompareDialog(QWidget, Ui_CompareDialog):
         self.contents_1.clear()
         self.contents_2.clear()
         
+        self.__generateFormats()
+        
         # counters for changes
         added = 0
         deleted = 0
@@ -450,7 +465,7 @@ class CompareWindow(E5MainWindow):
     """
     Main window class for the standalone dialog.
     """
-    def __init__(self, files=[], parent=None):
+    def __init__(self, files=None, parent=None):
         """
         Constructor
         
