@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2002 - 2017 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2002 - 2018 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -123,6 +123,10 @@ class FindFileDialog(QDialog, Ui_FindFileDialog):
         self.filterInterfaces = r'.*\.idl$'
         self.interfacesExt = ['*.idl']
         
+        # Protobuf protocol files
+        self.filterProtocols = r'.*\.proto$'
+        self.protocolsExt = ['*.proto']
+        
         # Qt resources files
         self.filterResources = r'.*\.qrc$'
         self.resourcesExt = ['*.qrc']
@@ -327,6 +331,8 @@ class FindFileDialog(QDialog, Ui_FindFileDialog):
                     files += self.project.pdata["FORMS"]
                 if self.interfacesCheckBox.isChecked():
                     files += self.project.pdata["INTERFACES"]
+                if self.protocolsCheckBox.isChecked():
+                    files += self.project.pdata["PROTOCOLS"]
                 if self.resourcesCheckBox.isChecked():
                     files += self.project.pdata["RESOURCES"]
         elif self.dirButton.isChecked():
@@ -338,11 +344,14 @@ class FindFileDialog(QDialog, Ui_FindFileDialog):
                             assoc.replace(".", "\.").replace("*", ".*"))
                          for assoc in list(
                              Preferences.getEditorLexerAssocs().keys())
-                         if assoc not in self.formsExt + self.interfacesExt])
+                         if assoc not in self.formsExt + self.interfacesExt +
+                            self.protocolsExt])
                 if self.formsCheckBox.isChecked():
                     filters.append(self.filterForms)
                 if self.interfacesCheckBox.isChecked():
                     filters.append(self.filterInterfaces)
+                if self.protocolsCheckBox.isChecked():
+                    filters.append(self.filterProtocols)
                 if self.resourcesCheckBox.isChecked():
                     filters.append(self.filterResources)
                 filterString = "|".join(filters)
