@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2013 - 2017 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2013 - 2018 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 # pylint: disable=C0103
 
@@ -93,7 +93,8 @@ class BackgroundService(QTcpServer):
             "Utilities", "BackgroundClient.py")
         proc = QProcess()
         proc.setProcessChannelMode(QProcess.ForwardedChannels)
-        args = [backgroundClient, self.hostAddress, str(port)]
+        args = [backgroundClient, self.hostAddress, str(port),
+                str(Preferences.getUI("BackgroundServiceProcesses"))]
         proc.start(interpreter, args)
         if not proc.waitForStarted(10000):
             proc = None
@@ -160,8 +161,10 @@ class BackgroundService(QTcpServer):
             
             packedData = b''
             while len(packedData) < length:
-                connection.waitForReadyRead(50)
-                packedData += connection.read(length - len(packedData))
+                maxSize = length - len(packedData)
+                if connection.bytesAvailable() < maxSize:
+                    connection.waitForReadyRead(50)
+                packedData += connection.read(maxSize)
 
             assert adler32(packedData) & 0xffffffff == datahash, \
                 'Hashes not equal'
@@ -208,7 +211,7 @@ class BackgroundService(QTcpServer):
                             'An error in Erics background client stopped the'
                             ' service.')
                         )
-                    except KeyError:
+                    except (KeyError, TypeError):
                         # ignore silently
                         pass
                 if res != E5MessageBox.No:
