@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2003 - 2017 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2003 - 2018 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -45,7 +45,7 @@ def __showwarning(message, category, filename, lineno, file=None, line=""):
     @param file file to write the warning message to (ignored)
     @param line line causing the warning (ignored)
     @raise SyntaxError
-    """
+    """     # __IGNORE_WARNING_D252__ __IGNORE_WARNING_D253__
     if category is SyntaxWarning:
         err = SyntaxError(str(message))
         err.filename = filename
@@ -65,14 +65,15 @@ from PyQt5.Qsci import QSCINTILLA_VERSION_STR, QsciScintilla
 from Globals import (  # __IGNORE_WARNING__
     isWindowsPlatform, isLinuxPlatform, isMacPlatform, getConfigDir,
     setConfigDir, getPythonModulesDirectory, getPyQt5ModulesDirectory,
-    getQtBinariesPath)
+    getQtBinariesPath, getPyQtToolsPath, qVersionTuple)
 
 from E5Gui.E5Application import e5App
 
 from UI.Info import Program, Version
 
 import Preferences
-from Plugins.CheckerPlugins.SyntaxChecker.SyntaxCheck import (  # __IGNORE_WARNING__
+from Plugins.CheckerPlugins.SyntaxChecker.SyntaxCheck import (
+    # __IGNORE_WARNING__
     normalizeCode)
 
 from eric6config import getConfig
@@ -201,10 +202,9 @@ def readEncodedFileWithHash(filename):
     f = open(filename, "rb")
     text = f.read()
     f.close()
-    hash = str(QCryptographicHash.hash(
-               QByteArray(text), QCryptographicHash.Md5).toHex(),
-               encoding="ASCII")
-    return decode(text) + (hash, )
+    hashStr = str(QCryptographicHash.hash(
+        QByteArray(text), QCryptographicHash.Md5).toHex(), encoding="ASCII")
+    return decode(text) + (hashStr, )
 
 
 def decode(text):
@@ -460,16 +460,16 @@ _escape_map = {
 }
 
 
-def escape_entities(m, map=_escape_map):
+def escape_entities(m, escmap=_escape_map):
     """
     Function to encode html entities.
     
     @param m the match object
-    @param map the map of entities to encode
+    @param escmap the map of entities to encode
     @return the converted text (string)
     """
     char = m.group()
-    text = map.get(char)
+    text = escmap.get(char)
     if text is None:
         text = "&#{0:d};".format(ord(char))
     return text
@@ -527,8 +527,8 @@ def unescape_uentities(m):
     @return the converted text (string)
     """
     char = m.group()
-    ord = int(char[2:-1])
-    return chr(ord)
+    ordinal = int(char[2:-1])
+    return chr(ordinal)
 
 
 def html_udecode(text, pattern=_uunescape):
@@ -656,7 +656,7 @@ def extractFlagsFromFile(filename):
     return extractFlags(source)
 
 
-def extractLineFlags(line, startComment="#", endComment=""):
+def extractLineFlags(line, startComment="#", endComment="", flagsLine=False):
     """
     Function to extract flags starting and ending with '__' from a line
     comment.
@@ -664,17 +664,22 @@ def extractLineFlags(line, startComment="#", endComment=""):
     @param line line to extract flags from (string)
     @keyparam startComment string identifying the start of the comment (string)
     @keyparam endComment string identifying the end of a comment (string)
+    @keyparam flagsLine flag indicating to check for a flags only line (bool)
     @return list containing the extracted flags (list of strings)
     """
     flags = []
     
-    pos = line.rfind(startComment)
-    if pos >= 0:
-        comment = line[pos + len(startComment):].strip()
-        if endComment:
-            comment = comment.replace("endComment", "")
-        flags = [f.strip() for f in comment.split()
-                 if (f.startswith("__") and f.endswith("__"))]
+    if not flagsLine or (
+       flagsLine and line.strip().startswith(startComment)):
+        pos = line.rfind(startComment)
+        if pos >= 0:
+            comment = line[pos + len(startComment):].strip()
+            if endComment:
+                endPos = line.rfind(endComment)
+                if endPos >= 0:
+                    comment = comment[:endPos]
+            flags = [f.strip() for f in comment.split()
+                     if (f.startswith("__") and f.endswith("__"))]
     return flags
 
 
@@ -820,8 +825,8 @@ def isinpath(file):
         return False
     
     dirs = path.split(os.pathsep)
-    for dir in dirs:
-        if os.access(os.path.join(dir, file), os.X_OK):
+    for directory in dirs:
+        if os.access(os.path.join(directory, file), os.X_OK):
             return True
     
     return False
@@ -937,8 +942,8 @@ def getExecutablePath(file):
         return ""
         
     dirs = path.split(os.pathsep)
-    for dir in dirs:
-        exe = os.path.join(dir, file)
+    for directory in dirs:
+        exe = os.path.join(directory, file)
         if os.access(exe, os.X_OK):
             return exe
             
@@ -972,8 +977,8 @@ def getExecutablePaths(file):
     # environment variable not defined
     if path is not None:
         dirs = path.split(os.pathsep)
-        for dir in dirs:
-            exe = os.path.join(dir, file)
+        for directory in dirs:
+            exe = os.path.join(directory, file)
             if os.access(exe, os.X_OK) and exe not in paths:
                 paths.append(exe)
     
@@ -1015,9 +1020,9 @@ def getWindowsExecutablePath(file):
         return ""
         
     dirs = path.split(os.pathsep)
-    for dir in dirs:
+    for directory in dirs:
         for filename in filenames:
-            exe = os.path.join(dir, filename)
+            exe = os.path.join(directory, filename)
             if os.access(exe, os.X_OK):
                 return exe
     
@@ -1032,6 +1037,24 @@ def isExecutable(exe):
     @return flag indicating executable status (boolean)
     """
     return os.access(exe, os.X_OK)
+
+
+def isDrive(path):
+    """
+    Function to check, if a path is a Windows drive.
+    
+    @param path path name to be checked
+    @type str
+    @return flag indicating a Windows drive
+    @rtype bool
+    """
+    isDrive = False
+    drive, directory = os.path.splitdrive(path)
+    if drive and len(drive) == 2 and drive.endswith(":") and \
+       directory in ["", "\\", "/"]:
+        isDrive = True
+    
+    return isDrive
     
 
 def samepath(f1, f2):
@@ -1570,12 +1593,12 @@ def getEnvironmentEntry(key, default=None):
     @return the requested entry or the default value, if the entry wasn't
         found (string or None)
     """
-    filter = QRegExp("^{0}[ \t]*=".format(key))
+    filterRe = QRegExp("^{0}[ \t]*=".format(key))
     if isWindowsPlatform():
-        filter.setCaseSensitivity(Qt.CaseInsensitive)
+        filterRe.setCaseSensitivity(Qt.CaseInsensitive)
     
     entries = [e for e in QProcess.systemEnvironment()
-               if filter.indexIn(e) != -1]
+               if filterRe.indexIn(e) != -1]
     if not entries:
         return default
     
@@ -1591,12 +1614,12 @@ def hasEnvironmentEntry(key):
     @param key key of the requested environment entry (string)
     @return flag indicating the presence of the requested entry (boolean)
     """
-    filter = QRegExp("^{0}[ \t]*=".format(key))
+    filterRe = QRegExp("^{0}[ \t]*=".format(key))
     if isWindowsPlatform():
-        filter.setCaseSensitivity(Qt.CaseInsensitive)
+        filterRe.setCaseSensitivity(Qt.CaseInsensitive)
     
     entries = [e for e in QProcess.systemEnvironment()
-               if filter.indexIn(e) != -1]
+               if filterRe.indexIn(e) != -1]
     return len(entries) > 0
 
 ###############################################################################
@@ -1666,7 +1689,42 @@ def prepareQtMacBundle(toolname, version, args):
     return ("open", newArgs)
 
 ###############################################################################
-# Qt utility functions below
+# PyQt utility functions below
+###############################################################################
+
+
+def generatePyQtToolPath(toolname, alternatives=None):
+    """
+    Module function to generate the executable path for a PyQt tool.
+    
+    @param toolname base name of the tool
+    @type str
+    @param alternatives list of alternative tool names to try
+    @type list of str
+    @return executable path name of the tool
+    @rtype str
+    """
+    pyqtVariant = int(toolname[-1])
+    pyqtToolsPath = getPyQtToolsPath(pyqtVariant)
+    if pyqtToolsPath:
+        exe = os.path.join(pyqtToolsPath, toolname)
+        if isWindowsPlatform():
+            exe += ".exe"
+    else:
+        if isWindowsPlatform():
+            exe = getWindowsExecutablePath(toolname)
+        else:
+            exe = toolname
+    
+    if not isinpath(exe) and alternatives:
+        ex_ = generatePyQtToolPath(alternatives[0], alternatives[1:])
+        if isinpath(ex_):
+            exe = ex_
+    
+    return exe
+
+###############################################################################
+# PySide utility functions below
 ###############################################################################
 
 
@@ -1674,7 +1732,7 @@ def generatePySideToolPath(toolname):
     """
     Module function to generate the executable path for a PySide tool.
     
-    @param toolname base name of the tool (string or QString)
+    @param toolname base name of the tool (string)
     @return the PySide tool path with extension (string)
     """
     if isWindowsPlatform():
@@ -1751,7 +1809,7 @@ def generateVersionInfo(linesep='\n'):
         qVersion(), linesep, PYQT_VERSION_STR, linesep)
     info += "  sip {0}{1}  QScintilla {2}{3}".format(
         sip_version_str, linesep, QSCINTILLA_VERSION_STR, linesep)
-    if qVersion() >= "5.6.0":
+    if qVersionTuple() >= (5, 6, 0):
         try:
             from PyQt5 import QtWebEngineWidgets    # __IGNORE_WARNING__
             from WebBrowser.Tools import WebBrowserTools
