@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2009 - 2016 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2009 - 2018 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -55,22 +55,28 @@ class HistoryMenuModel(QAbstractProxyModel):
             return 0
         return min(self.__treeModel.rowCount(first), self.MOVEDROWS)
     
-    def columnCount(self, parent=QModelIndex()):
+    def columnCount(self, parent=None):
         """
         Public method to get the number of columns.
         
         @param parent index of parent (QModelIndex)
         @return number of columns (integer)
         """
+        if parent is None:
+            parent = QModelIndex()
+        
         return self.__treeModel.columnCount(self.mapToSource(parent))
     
-    def rowCount(self, parent=QModelIndex()):
+    def rowCount(self, parent=None):
         """
         Public method to determine the number of rows.
         
         @param parent index of parent (QModelIndex)
         @return number of rows (integer)
         """
+        if parent is None:
+            parent = QModelIndex()
+        
         if parent.column() > 0:
             return 0
         
@@ -133,7 +139,7 @@ class HistoryMenuModel(QAbstractProxyModel):
         treeIndex = self.__treeModel.mapFromSource(historyIndex)
         return treeIndex
     
-    def index(self, row, column, parent=QModelIndex()):
+    def index(self, row, column, parent=None):
         """
         Public method to create an index.
         
@@ -142,6 +148,9 @@ class HistoryMenuModel(QAbstractProxyModel):
         @param parent index of the parent item (QModelIndex)
         @return requested index (QModelIndex)
         """
+        if parent is None:
+            parent = QModelIndex()
+        
         if row < 0 or \
            column < 0 or \
            column >= self.columnCount(parent) or \
@@ -279,6 +288,7 @@ class HistoryMenu(E5ModelMenu):
         E5ModelMenu.__init__(self, parent)
         
         self.__tabWidget = tabWidget
+        self.__mw = parent
         
         self.__historyManager = None
         self.__historyMenuModel = None
@@ -362,7 +372,7 @@ class HistoryMenu(E5ModelMenu):
         
         act = self.addAction(UI.PixmapCache.getIcon("history.png"),
                              self.tr("Show All History..."))
-        act.triggered.connect(self.__showHistoryDialog)
+        act.triggered.connect(self.showHistoryDialog)
         act = self.addAction(UI.PixmapCache.getIcon("historyClear.png"),
                              self.tr("Clear History..."))
         act.triggered.connect(self.__clearHistoryDialog)
@@ -378,12 +388,12 @@ class HistoryMenu(E5ModelMenu):
         for act in self.__initialActions:
             self.addAction(act)
     
-    def __showHistoryDialog(self):
+    def showHistoryDialog(self):
         """
-        Private slot to show the history dialog.
+        Public slot to show the history dialog.
         """
         from .HistoryDialog import HistoryDialog
-        dlg = HistoryDialog(self)
+        dlg = HistoryDialog(self.__mw)
         dlg.openUrl.connect(self.openUrl)
         dlg.newTab.connect(self.newTab)
         dlg.newBackgroundTab.connect(self.newBackgroundTab)
