@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2008 - 2017 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2008 - 2018 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 
@@ -16,8 +16,9 @@ except NameError:
 
 import os
 
-from PyQt5.QtCore import pyqtSignal, QUrl, QFileInfo, Qt, QTimer, QEvent, \
-    QPoint, QDateTime, qVersion, QStandardPaths
+from PyQt5.QtCore import pyqtSignal, PYQT_VERSION, Qt, QUrl, QFileInfo, \
+    QTimer, QEvent, QPoint, QPointF, QDateTime, QStandardPaths, QByteArray, \
+    QIODevice, QDataStream
 from PyQt5.QtGui import QDesktopServices, QClipboard, QIcon, \
     QContextMenuEvent, QPixmap
 from PyQt5.QtWidgets import qApp, QStyle, QMenu, QApplication
@@ -38,6 +39,7 @@ from .Tools.WebBrowserTools import readAllFileContents, pixmapToDataUrl
 import Preferences
 import UI.PixmapCache
 import Utilities
+from Globals import qVersionTuple
 
 
 class WebBrowserView(QWebEngineView):
@@ -51,6 +53,10 @@ class WebBrowserView(QWebEngineView):
     @signal search(QUrl) emitted, when a search is requested
     @signal zoomValueChanged(int) emitted to signal a change of the zoom value
     @signal faviconChanged() emitted to signal a changed web site icon
+    @signal safeBrowsingAbort() emitted to indicate an abort due to a safe
+        browsing event
+    @signal safeBrowsingBad(threatType, threatMessages) emitted to indicate a
+        malicious web site as determined by safe browsing
     """
     sourceChanged = pyqtSignal(QUrl)
     forwardAvailable = pyqtSignal(bool)
@@ -59,6 +65,8 @@ class WebBrowserView(QWebEngineView):
     search = pyqtSignal(QUrl)
     zoomValueChanged = pyqtSignal(int)
     faviconChanged = pyqtSignal()
+    safeBrowsingAbort = pyqtSignal()
+    safeBrowsingBad = pyqtSignal(str, str)
     
     ZoomLevels = [
         30, 40, 50, 67, 80, 90,
@@ -83,8 +91,8 @@ class WebBrowserView(QWebEngineView):
         
         self.__speedDial = WebBrowserWindow.speedDial()
         
-        self.__page = WebBrowserPage(self)
-        self.setPage(self.__page)
+        self.__page = None
+        self.__createNewPage()
         
         self.__mw = mainWindow
         self.__isLoading = False
@@ -122,7 +130,9 @@ class WebBrowserView(QWebEngineView):
         self.__inspector = None
         WebInspector.registerView(self)
         
-        if qVersion() >= "5.8.0":
+        self.__restoreData = None
+        
+        if qVersionTuple() >= (5, 8, 0):
             if self.parentWidget() is not None:
                 self.parentWidget().installEventFilter(self)
             
@@ -133,6 +143,16 @@ class WebBrowserView(QWebEngineView):
         
         self.grabGesture(Qt.PinchGesture)
     
+    def __createNewPage(self):
+        """
+        Private method to create a new page object.
+        """
+        self.__page = WebBrowserPage(self)
+        self.setPage(self.__page)
+        
+        self.__page.safeBrowsingAbort.connect(self.safeBrowsingAbort)
+        self.__page.safeBrowsingBad.connect(self.safeBrowsingBad)
+    
     def __setRwhvqt(self):
         """
         Private slot to set widget that receives input events.
@@ -342,9 +362,10 @@ class WebBrowserView(QWebEngineView):
         """
         Public slot to clear the current selection.
         """
-        if qVersion() >= "5.7.0":
+        try:
             self.triggerPageAction(QWebEnginePage.Unselect)
-        else:
+        except AttributeError:
+            # prior to 5.7.0
             self.page().runJavaScript(
                 "window.getSelection().empty()",
                 WebBrowserPage.SafeJsWorld)
@@ -525,11 +546,11 @@ class WebBrowserView(QWebEngineView):
         @type WebHitTestResult
         """
         spellCheckActionCount = 0
-        if qVersion() >= "5.7.0":
+        if qVersionTuple() >= (5, 7, 0) and PYQT_VERSION >= 0x50700:
             contextMenuData = self.page().contextMenuData()
             hitTest.updateWithContextMenuData(contextMenuData)
             
-            if qVersion() >= "5.8.0" and \
+            if qVersionTuple() >= (5, 8, 0) and PYQT_VERSION >= 0x50800 and \
                bool(contextMenuData.misspelledWord()):
                 boldFont = menu.font()
                 boldFont.setBold(True)
@@ -540,7 +561,7 @@ class WebBrowserView(QWebEngineView):
                         self.__replaceMisspelledWord)
                     act.setFont(boldFont)
                 
-                if bool(menu.actions()):
+                if not bool(menu.actions()):
                     menu.addAction(self.tr("No suggestions")).setEnabled(False)
                 
                 menu.addSeparator()
@@ -781,7 +802,7 @@ class WebBrowserView(QWebEngineView):
             language = languages[0]
             langCode = language.split("[")[1][:2]
             googleTranslatorUrl = QUrl(
-                "http://translate.google.com/#auto|{0}|{1}".format(
+                "http://translate.google.com/#auto/{0}/{1}".format(
                     langCode, self.selectedText()))
             menu.addAction(
                 UI.PixmapCache.getIcon("translate.png"),
@@ -1287,8 +1308,8 @@ class WebBrowserView(QWebEngineView):
         if self.__mw and self.__mw.isFullScreen():
             if self.__mw.isFullScreenNavigationVisible():
                 self.__mw.hideFullScreenNavigation()
-            elif evt.y() < 5:
-                # mouse is within 5px to the top
+            elif evt.y() < 10:
+                # mouse is within 10px to the top
                 self.__mw.showFullScreenNavigation()
         
         if WebBrowserWindow.autoScroller().mouseMove(evt):
@@ -1391,7 +1412,7 @@ class WebBrowserView(QWebEngineView):
         @rtype bool
         """
         # find the render widget receiving events for the web page
-        if qVersion() < "5.8.0":
+        if qVersionTuple() < (5, 8, 0):
             if obj is self and evt.type() == QEvent.ChildAdded:
                 child = evt.child()
                 if child and child.inherits(
@@ -1575,8 +1596,7 @@ class WebBrowserView(QWebEngineView):
         @type QWebEnginePage.RenderProcessTerminationStatus
         """
         self.page().deleteLater()
-        self.__page = WebBrowserPage(self)
-        self.setPage(self.__page)
+        self.__createNewPage()
         
         html = readAllFileContents(":/html/tabCrashPage.html")
         html = html.replace("@IMAGE@", pixmapToDataUrl(
@@ -1704,7 +1724,7 @@ class WebBrowserView(QWebEngineView):
         if url.isEmpty():
             return
         
-        if qVersion() >= "5.8.0":
+        if qVersionTuple() >= (5, 8, 0) and PYQT_VERSION >= 0x50800:
             # since Qt 5.8.0
             fileName, savePageFormat = self.__getSavePageFileNameAndFormat()
             if fileName:
@@ -1893,3 +1913,171 @@ class WebBrowserView(QWebEngineView):
         Private slot to reset all speed dials to the default pages.
         """
         self.__speedDial.resetDials()
+    
+    ###########################################################################
+    ## Methods below implement session related functions
+    ###########################################################################
+    
+    def storeSessionData(self, data):
+        """
+        Public method to store session data to be restored later on.
+        
+        @param data dictionary with session data to be restored
+        @type dict
+        """
+        self.__restoreData = data
+    
+    def __showEventSlot(self):
+        """
+        Private slot to perform actions when the view is shown and the event
+        loop is running.
+        """
+        if self.__restoreData:
+            sessionData, self.__restoreData = self.__restoreData, None
+            self.loadFromSessionData(sessionData)
+    
+    def showEvent(self, evt):
+        """
+        Protected method to handle show events.
+        
+        @param evt reference to the show event object
+        @type QShowEvent
+        """
+        super(WebBrowserView, self).showEvent(evt)
+        self.activateSession()
+    
+    def activateSession(self):
+        """
+        Public slot to activate a restored session.
+        """
+        if self.__restoreData and not self.__mw.isClosing():
+            QTimer.singleShot(0, self.__showEventSlot)
+    
+    def getSessionData(self):
+        """
+        Public method to populate the session data.
+        
+        @return dictionary containing the session data
+        @rtype dict
+        """
+        if self.__restoreData:
+            # page has not been shown yet
+            return self.__restoreData
+        
+        sessionData = {}
+        page = self.page()
+        
+        # 1. zoom factor
+        sessionData["ZoomFactor"] = page.zoomFactor()
+        
+        # 2. scroll position
+        scrollPos = page.scrollPosition()
+        sessionData["ScrollPosition"] = {
+            "x": scrollPos.x(),
+            "y": scrollPos.y(),
+        }
+        
+        # 3. page history
+        historyArray = QByteArray()
+        stream = QDataStream(historyArray, QIODevice.WriteOnly)
+        stream << page.history()
+        sessionData["History"] = str(
+            historyArray.toBase64(QByteArray.Base64UrlEncoding),
+            encoding="ascii")
+        sessionData["HistoryIndex"] = page.history().currentItemIndex()
+        
+        # 4. current URL and title
+        sessionData["Url"] = self.url().toString()
+        sessionData["Title"] = self.title()
+        
+        # 5. web icon
+        iconArray = QByteArray()
+        stream = QDataStream(iconArray, QIODevice.WriteOnly)
+        stream << page.icon()
+        sessionData["Icon"] = str(iconArray.toBase64(), encoding="ascii")
+        
+        return sessionData
+    
+    def loadFromSessionData(self, sessionData):
+        """
+        Public method to load the session data.
+        
+        @param sessionData dictionary containing the session data as
+            generated by getSessionData()
+        @type dict
+        """
+        page = self.page()
+        # blank the page
+        page.setUrl(QUrl("about:blank"))
+        
+        # 1. page history
+        if "History" in sessionData:
+            historyArray = QByteArray.fromBase64(
+                sessionData["History"].encode("ascii"),
+                QByteArray.Base64UrlEncoding)
+            stream = QDataStream(historyArray, QIODevice.ReadOnly)
+            stream >> page.history()
+            
+            if "HistoryIndex" in sessionData:
+                item = page.history().itemAt(sessionData["HistoryIndex"])
+                if item is not None:
+                    page.history().goToItem(item)
+        
+        # 2. zoom factor
+        if "ZoomFactor" in sessionData:
+            page.setZoomFactor(sessionData["ZoomFactor"])
+        
+        # 3. scroll position
+        if "ScrollPosition" in sessionData:
+            scrollPos = sessionData["ScrollPosition"]
+            page.scrollTo(QPointF(scrollPos["x"], scrollPos["y"]))
+    
+    def extractSessionMetaData(self, sessionData):
+        """
+        Public method to extract some session meta data elements needed by the
+        tab widget in case of deferred loading.
+        
+        @param sessionData dictionary containing the session data as
+            generated by getSessionData()
+        @type dict
+        @return tuple containing the title, URL and web icon
+        @rtype tuple of (str, str, QIcon)
+        """
+        if "Title" in sessionData:
+            title = sessionData["Title"]
+        else:
+            title = ""
+        
+        if "Url" in sessionData:
+            urlStr = sessionData["Url"]
+        else:
+            urlStr = ""
+        
+        if "Icon" in sessionData:
+            iconArray = QByteArray.fromBase64(
+                sessionData["Icon"].encode("ascii"))
+            stream = QDataStream(iconArray, QIODevice.ReadOnly)
+            icon = QIcon()
+            stream >> icon
+        else:
+            from .Tools import WebIconProvider
+            icon = WebIconProvider.instance().iconForUrl(
+                QUrl.fromUserInput(urlStr))
+        
+        return title, urlStr, icon
+    
+    ###########################################################################
+    ## Methods below implement safe browsing related functions
+    ###########################################################################
+    
+    def getSafeBrowsingStatus(self):
+        """
+        Public method to get the safe browsing status of the current page.
+        
+        @return flag indicating a safe site
+        @rtype bool
+        """
+        if self.__page:
+            return self.__page.getSafeBrowsingStatus()
+        else:
+            return True
