@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2002 - 2017 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2002 - 2018 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -144,7 +144,6 @@ def excepthook(excType, excValue, tracebackobj):
     @param excValue exception value
     @param tracebackobj traceback object
     """
-    import xml.sax.saxutils
     from UI.Info import BugAddress
     import Utilities
     import Globals
@@ -197,7 +196,10 @@ def excepthook(excType, excValue, tracebackobj):
     else:
         warning = notice + msg + versionInfo
         # Escape &<> otherwise it's not visible in the error dialog
-        warning = xml.sax.saxutils.escape(warning)
+        warning = warning\
+            .replace("&", "&amp;")\
+            .replace(">", "&gt;")\
+            .replace("<", "&lt;")
         if sys.version_info[0] == 2:
             warning = warning.encode('utf-8', 'replace')
         qWarning(warning)
